/*
 * test_text.c
 *
 * Copyright 2022 Jonathan Schöbel <jonathan@Ubermos-2019>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 *
 *
 */


#include <check.h>
#include <limits.h>
#include <stdbool.h>
#include <stdlib.h>

#include "macro.h"
#include "error.h"

#include "text.h"


START_TEST(test_text)
{
	struct SH_Error error;
	struct SH_Text * text;

	text = SH_Text_new (NULL);
	ck_assert_int_ne ((long int) text, (long int) NULL);

	ck_assert_int_ge  (text->size, CHUNK_SIZE);

	SH_Text_free (text, NULL);

	error.type = UNDEFINED;
	text = SH_Text_new (&error);
	ck_assert_int_ne ((long int) text, (long int) NULL);
	ck_assert_int_eq (error.type, SUCCESS);

	ck_assert_int_ge  (text->size, CHUNK_SIZE);

	error.type = UNDEFINED;
	SH_Text_free (text, &error);
	ck_assert_int_eq (error.type, SUCCESS);
}
END_TEST

START_TEST(test_text_enlarge)
{
	struct SH_Error error;
	struct SH_Text * text;
	bool result;
	size_t size1;
	size_t size2;
	size_t size2_;
	size_t size3;


	size1 = CHUNK_SIZE;
	size2 = CHUNK_SIZE + 1;
	size2_ = 2 * CHUNK_SIZE;
	if (SIZE_MAX % CHUNK_SIZE == 0)
	{
		size3 = ((SIZE_MAX / CHUNK_SIZE) - 1) * CHUNK_SIZE;
	}
	else
	{
		size3 = (SIZE_MAX / CHUNK_SIZE) * CHUNK_SIZE;
	}

	text = SH_Text_new (NULL);

	/* size == current size */
	result = SH_Text_enlarge (text, size1, NULL);
	ck_assert_int_eq (result, TRUE);

	ck_assert_int_eq  (text->size, size1);

	/* size > current size */
	result = SH_Text_enlarge (text, size2, NULL);
	ck_assert_int_eq (result, TRUE);

	ck_assert_int_eq (text->size, size2_);

	/* size < current size */
	result = SH_Text_enlarge (text, size1, NULL);
	ck_assert_int_eq (result, TRUE);

	ck_assert_int_eq (text->size, size2_);

	/* size throw overflow */
	result = SH_Text_enlarge (text, size3, NULL);
	ck_assert_int_eq (result, FALSE);

	SH_Text_free (text, NULL);


	text = SH_Text_new (NULL);

	/* size == current size */
	error.type = UNDEFINED;
	result = SH_Text_enlarge (text, size1, &error);
	ck_assert_int_eq (result, TRUE);
	ck_assert_int_eq (error.type, SUCCESS);

	ck_assert_int_eq  (text->size, size1);

	/* size > current size */
	error.type = UNDEFINED;
	result = SH_Text_enlarge (text, size2, &error);
	ck_assert_int_eq (result, TRUE);
	ck_assert_int_eq (error.type, SUCCESS);

	ck_assert_int_eq (text->size, size2_);

	/* size < current size */
	error.type = UNDEFINED;
	result = SH_Text_enlarge (text, size1, &error);
	ck_assert_int_eq (result, TRUE);
	ck_assert_int_eq (error.type, SUCCESS);

	ck_assert_int_eq (text->size, size2_);

	/* size throw overflow */
	error.type = UNDEFINED;
	result = SH_Text_enlarge (text, size3, &error);
	ck_assert_int_eq (result, FALSE);
	ck_assert_int_eq (error.type, DOMAIN_ERROR);

	SH_Text_free (text, NULL);
}
END_TEST

Suite * text_suite (void)
{
	Suite *s;
	TCase *tc_core;

	s = suite_create ("Testsuite SeFHT Text");

	/* Core test case */
	tc_core = tcase_create ("Core");

	tcase_add_test (tc_core, test_text);
	tcase_add_test (tc_core, test_text_enlarge);
	suite_add_tcase (s, tc_core);

	return s;
}

int main (int argc, char **argv)
{
	int number_failed;
	Suite *s;
	SRunner *sr;

	s = text_suite ();
	sr = srunner_create (s);

	srunner_run_all (sr, CK_NORMAL);
	number_failed = srunner_ntests_failed (sr);
	srunner_free (sr);

	return (number_failed == 0) ? EXIT_SUCCESS : EXIT_FAILURE;
}

